@@ -15,7 +15,7 @@ def init(version = None):
     register_svgcomm()
 
     if version is None:
-        igv_filepath = os.path.join(os.path.dirname(__file__), 'js/igv.min.js')
+        igv_filepath = os.path.join(os.path.dirname(__file__), 'js/igv.iife.js')
         igv_file = open(igv_filepath, 'r')
         igv_js = igv_file.read()
     else:
@@ -61,7 +61,8 @@ class Browser(object):
         Create an igv.js "Browser" instance on the front end.
         Retain a DisplayHandle for later updates (e.g. to convert browser to SVG)
         """
-        self.d = display(HTML("""<div id="%s"></div>""" % (self.igv_id)), display_id=id)
+
+        self.d = display(HTML(f'<div id="{self.igv_id}_buttons"></div><div id="{self.igv_id}_igvcontainer"></div>'), display_id=id)
 
         self._send({
             "id": self.igv_id,
@@ -69,6 +70,40 @@ class Browser(object):
             "data": config
         })
 
+    def load_session(self,session=None, url=None, path=None ):
+
+        """
+        Load a session.  Corresponds to the igv.js Browser function loadSession (see https://github.com/igvteam/igv.js/wiki/Browser-Control-2.0#loadtrack).
+        :param url: A url to na igv session file
+        :param path: A local file path to an igv session file
+        :param json: A dictionary representing an igv session object
+        :return:
+        """
+
+       # Check for minimal requirements
+       # if isinstance(config, dict) == False:
+       #     if isinstance(config, str):
+       #         config = {"url": config}
+       #     else:
+       #         raise Exception("load_track parameter must be a dictionary or url (string) to a track data file")
+
+
+        if path is not None:
+            with open(path) as user_file:
+                session = json.load(user_file)
+
+        elif url is not None:
+            session = requests.get(url).json()
+
+        if session is not None:
+            self._send({
+                "id": self.igv_id,
+                "command": "loadSession",
+                "data": session
+            })
+
+
+
     def load_track(self, config):
         """
         Load a track.  Corresponds to the igv.js Browser function loadTrack (see https://github.com/igvteam/igv.js/wiki/Browser-Control-2.0#loadtrack).
@@ -90,6 +125,8 @@ class Browser(object):
             "data": config
         })
 
+
+
     def load_roi(self, config):
         """
         Load regions of interest.  Corresponds to the igv.js Browser function loadROI (see https://github.com/igvteam/igv.js/wiki/Browser-API-2.0#loadroiconfigorarray).
